@@ -7,6 +7,7 @@ import tempfile
 from PIL import Image
 from io import BytesIO
 from pathlib import Path
+import json
 
 class BasketCase:
     def __init__(self, session_id=None):
@@ -21,46 +22,59 @@ class BasketCase:
 
     def fetch(self, target_urls):
         urls = self._scan(target_urls)
-        
+
         if urls['images'] or urls['videos']:
             self._prepare_output()
 
             for url in urls['images']:
                 self._get_image(url)
-
+            
             self._get_videos(urls['videos'])
         else:
-            print('Nothing was found. Maybe you need a session cookie?')
+            print('Nothing to download.')
 
     def _prepare_output(self):
         Path(self._output_images).mkdir(parents=True, exist_ok=True)
         Path(self._output_videos).mkdir(parents=True, exist_ok=True)
 
     def _scan(self, target_urls):
-        sets = {'images': set(), 'videos': set()}
+        sets = {
+            'images': set(),
+            'videos': set()
+        }
 
         print('Scanning the targets. This can take a while.')
 
         for target_url in target_urls:
-            request = self._session.get(target_url, timeout=10)
-            image_urls = re.findall(r'"display_url":"(.*?)"', request.text)
-            video_urls = re.findall(r'"video_url":"(.*?)"', request.text)
+            json_string = None
 
-            for image_url in image_urls:
-                image_url = self._decode_ampersands(image_url)
-                sets['images'].add(image_url)
+            with self._session.get(target_url, timeout=10) as request:
+                json_string = re.search(r'window\.__additionalDataLoaded\s*\(\s*[^,]+,\s*({.+?})\s*\);', request.text)
 
-            for video_url in video_urls:
-                video_url = self._decode_ampersands(video_url)
-                sets['videos'].add(video_url)
+                if not json_string:
+                    raise RuntimeError(f'JSON string not found at {target_url}')
+
+            json_data = json.loads(json_string.group(1))
+
+            if 'carousel_media' in json_data['items'][0]:
+                for media in json_data['items'][0]['carousel_media']:
+                    sets['images'].add(media['image_versions2']['candidates'][0]['url'])
+
+                    if 'video_versions' in media:
+                        sets['videos'].add(media['video_versions'][0]['url'])
+            else:
+                if 'image_versions2' not in json_data['items'][0]:
+                    raise RuntimeError(f'Unrecognized JSON structure at {target_url}')
+                
+                sets['images'].add(json_data['items'][0]['image_versions2']['candidates'][0]['url'])
+
+                if 'video_versions' in json_data['items'][0]:
+                    sets['videos'].add(json_data['items'][0]['video_versions'][0]['url'])
 
         return sets
 
-    def _decode_ampersands(self, url):
-        return re.sub(r'\\u0026', '&', url)
-
     def _get_image(self, url):
-        print('Downloading image:', url)
+        print(f'Downloading image: {url}')
 
         request = self._session.get(url, timeout=10)
 
